#include <windows.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <process.h>
#include "packet32.h"


#ifdef DHCPCD_EXPORTS
#define DHCPCD_API __declspec(dllexport)
#else
#define DHCPCD_API __declspec(dllimport)
#endif

/* UDP port numbers for DHCP */
#define	DHCP_SERVER_PORT	67	/* from client to server */
#define DHCP_CLIENT_PORT	68	/* from server to client */

/* DHCP message OP code */
#define DHCP_BOOTREQUEST	1
#define DHCP_BOOTREPLY		2

/* DHCP message type */
#define	DHCP_DISCOVER		1
#define DHCP_OFFER			2
#define	DHCP_REQUEST		3
#define	DHCP_DECLINE		4
#define	DHCP_ACK			5
#define DHCP_NAK			6
#define	DHCP_RELEASE		7
#define DHCP_INFORM			8
/* DHCP RETRANSMISSION TIMEOUT (microseconds) */
#define DHCP_INITIAL_RTO    ( 4*1000000)
#define DHCP_MAX_RTO        (64*1000000)

// Hardware Type: Ethernet
#define ARPHRD_ETHER		1	
#define BROADCAST_FLAG		0x8000
#define MAGIC_COOKIE		0x63825363

#define Max_Num_Adapter 10
#define ETH_ALEN		6		/* Octets in one ethernet addr	 */
#define	MAX_ETH_SIZE	1514
#define RECV_BUF		512000
#define ETH_P_IP		0x0800		/* Internet Protocol packet	*/
#define UDP				0x11

#define SIZE_OF_UDPHDR	8	// sizeof(struct udphdr);	//8
#define SIZE_OF_IPHDR	20	// sizeof(struct iphdr);	//20
#define SIZE_OF_ETHHDR	14	// sizeof(struct ethhdr);	//14

typedef unsigned int	__u32;
typedef unsigned short	__u16;
typedef unsigned char	__u8;

/* DHCP option and value (cf. RFC1533) */
enum
{
	padOption				=	0,
	subnetMask				=	1,
	timerOffset				=	2,
	routersOnSubnet			=	3,
	timeServer				=	4,
	nameServer				=	5,
	dns						=	6,
	logServer				=	7,
	cookieServer			=	8,
	lprServer				=	9,
	impressServer			=	10,
	resourceLocationServer	=	11,
	hostName				=	12,
	bootFileSize			=	13,
	meritDumpFile			=	14,
	domainName				=	15,
	swapServer				=	16,
	rootPath				=	17,
	extentionsPath			=	18,
	IPforwarding			=	19,
	nonLocalSourceRouting	=	20,
	policyFilter			=	21,
	maxDgramReasmSize		=	22,
	defaultIPTTL			=	23,
	pathMTUagingTimeout		=	24,
	pathMTUplateauTable		=	25,
	ifMTU					=	26,
	allSubnetsLocal			=	27,
	broadcastAddr			=	28,
	performMaskDiscovery	=	29,
	maskSupplier			=	30,
	performRouterDiscovery	=	31,
	routerSolicitationAddr	=	32,
	staticRoute				=	33,
	trailerEncapsulation	=	34,
	arpCacheTimeout			=	35,
	ethernetEncapsulation	=	36,
	tcpDefaultTTL			=	37,
	tcpKeepaliveInterval	=	38,
	tcpKeepaliveGarbage		=	39,
	nisDomainName			=	40,
	nisServers				=	41,
	ntpServers				=	42,
	vendorSpecificInfo		=	43,
	netBIOSnameServer		=	44,
	netBIOSdgramDistServer	=	45,
	netBIOSnodeType			=	46,
	netBIOSscope			=	47,
	xFontServer				=	48,
	xDisplayManager			=	49,
	dhcpRequestedIPaddr		=	50,
	dhcpIPaddrLeaseTime		=	51,
	dhcpOptionOverload		=	52,
	dhcpMessageType			=	53,
	dhcpServerIdentifier	=	54,
	dhcpParamRequest		=	55,
	dhcpMsg					=	56,
	dhcpMaxMsgSize			=	57,
	dhcpT1value				=	58,
	dhcpT2value				=	59,
	dhcpClassIdentifier		=	60,
	dhcpClientIdentifier	=	61,
	endOption				=	255
};

struct ethhdr 
{
	unsigned char	h_dest[ETH_ALEN];	/* destination eth addr	*/
	unsigned char	h_source[ETH_ALEN];	/* source ether addr	*/
	unsigned short	h_proto;			/* packet type ID field	*/
};


struct iphdr 
{
	__u8	ihl:4, 
		version:4;	// LITTLE_ENDIAN_BITFIELD ( for INTEL chip )
	__u8	tos;
	__u16	tot_len;
	__u16	id;
	__u16	frag_off;
	__u8	ttl;
	__u8	protocol;
	__u16	check;
	__u32	saddr;
	__u32	daddr;
};

struct udphdr {
	__u16	source;
	__u16	dest;
	__u16	len;
	__u16	check;
};

struct pesudeIphdr
{
	__u32	saddr;
	__u32	daddr;
	__u8	zero;		// should be 0
	__u8	protocol;
	__u16	len;		// UDP/TCP data length
};

struct dhcpMessage
{
	__u8	op;			/* message type */
	__u8	htype;		/* hardware address type */
	__u8	hlen;		/* hardware address length */
	__u8	hops;		/* should be zero in client's message */
	__u32   xid;		/* transaction id */
	__u16	secs;		/* elapsed time in sec. from trying to boot */
	__u16	flags;
	__u32   ciaddr;		/* (previously allocated) client IP address */
	__u32	yiaddr;		/* 'your' client IP address */
	__u32	siaddr;		/* should be zero in client's messages */
	__u32	giaddr;		/* should be zero in client's messages */
	__u8	chaddr[16];	/* client's hardware address */
	__u8	sname[64];	/* server host name, null terminated string */
	__u8	file[128];	/* boot file name, null terminated string */
	__u8	options[312];/* message options */
};

typedef struct dhcpOptions
{
	__u8 num;
	__u8 len;
	char val[256];
}  dhcpOptions;


struct dhcpGetInfo
{
	__u32				xid;					
	__u32				yourClientIPAddress;	
	__u32				nextServerIPAddress;	
	unsigned char		subnetMask[4];			
	unsigned char		defaultGateway[4];		
	unsigned char		dns[4];					
	unsigned char		domainName[255];	
	unsigned char		leaseTime[4];			
	unsigned char		dhcpMessageType[1];		
	unsigned char		dhcpServerIdentifier[4];

	__u32				dhcpLeaseTime;				
	__u32				dhcpT1value;
	__u32				dhcpT2value;

	bool				dhcpDemonStatus;
};




class DHCPCD_API CDhcpcd
{
public:
	CDhcpcd::CDhcpcd(
		char	*myAddress,
		char	*dhcpID);

	bool start(struct dhcpGetInfo *_dhcpGetInfo);
	bool status(struct dhcpGetInfo *_dhcpGetInfo);
	bool end();
	
	bool dhcpClientDemon();

	~CDhcpcd();

	bool dhcpcdTempRecvThread();

private:


	bool buildDiscoverMsg(char *_packetMessage);

	bool parseDhcpMsgRecv(
		struct dhcpMessage *_dhcpMsg,				// [in]
		struct dhcpGetInfo *_dhcpGetInfo			// [out]
	);

	bool packetDelimiter(struct dhcpGetInfo *_dhcpGetInfo);
	
	bool buildRequestMsg(
		char *_packetMessage,
		struct dhcpGetInfo *_dhcpGetInfo);
	
	bool analizeRawPacket(
		char* pChar, 
		int dataLen,
		struct dhcpGetInfo *_dhcpGetInfo);

	struct dhcpGetInfo m_dhcpGetInfo;

	bool			m_CDhcpFlag;
	char			m_message[1024];
	unsigned char	m_macAddr[ETH_ALEN];
	char			m_ipAddr[16];
	char			m_maskAddr[16];
	char			m_AdapterName[512];

	LPADAPTER		m_sendLpAdapter;
	LPPACKET		m_sendLpPacket;
	char			m_sendBuffer[MAX_ETH_SIZE];

	LPADAPTER		m_recvLpAdapter;
	LPPACKET		m_recvLpPacket;
	char			m_recvBuffer[256000];

	__u32			m_xid;
	char			m_dhcpID[255];
	__u32			m_waitTime;
	HANDLE			m_dhcpcdThreadHandle;
	unsigned		m_dhcpcdThreadID;

	unsigned char	m_waitingPacketType;
	HANDLE			m_dhcpcdLock_out,m_dhcpcdLock_in;
	HANDLE			m_dhcpcdTempRecvThreadHandle;
	unsigned		m_dhcpcdTempRecvThreadID;


};

		
